<?php
/**
 * Forcaster for Elementor
 * The Forcaster is a simple and intuitive WordPress plugin of the weather forecast for the coming days or hours.
 * Exclusively on https://1.envato.market/forcaster-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.7
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\ForcasterElementor;

use Merkulove\ForcasterElementor\Unity\Plugin;
use Merkulove\ForcasterElementor\Unity\Settings;
use Merkulove\ForcasterElementor\Unity\TabGeneral;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 *
 * @since 1.0.0
 *
 **/
final class Config {

	/**
	 * The one true Settings.
	 *
     * @since 1.0.0
     * @access private
	 * @var Config
	 **/
	private static $instance;

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

        /** Get default plugin settings. */
        $tabs = Plugin::get_tabs();

        /** Short hand access to plugin settings. */
        $options = Settings::get_instance()->options;

        $offset = 0;
        $tabs = array_slice( $tabs, 0, $offset, true ) +
            [ 'integrations' => [
                'enabled'       => true,
                'class'         => TabGeneral::class,
                'label'         => esc_html__( 'Integrations', 'forcaster-elementor' ),
                'title'         => esc_html__( 'OpenWeather Settings', 'forcaster-elementor' ),
                'show_title'    => true,
                'icon'          => 'api',
                'fields'        => []
            ] ] +
            array_slice( $tabs, $offset, NULL, true );


        $tabs[ 'integrations' ][ 'fields' ][ 'mdp_synopter_elementor_weather_settings' ] = [
            'type'              => 'text',
            'label'             => esc_html__( 'OpenWeather Key', 'forcaster-elementor' ),
            'placeholder'       => esc_html__( 'OpenWeather Key', 'forcaster-elementor' ),
            'description'       => esc_html__( 'You can create key by clicking on the link', 'forcaster-elementor' ) . ' <a href="https://home.openweathermap.org/users/sign_up" target="_blank">Open Weather</a>',
            'default'           => '',
            'attr'              => [
                'maxlength' => '250'
            ]
        ];

        unset( $tabs['general'] );

        /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->get_options();

    }

	/**
	 * Main Settings Instance.
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Config
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
