<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * forcaster for Elementor
 * Weekly, Daily and Hourly weather forecast for Elementor
 * Exclusively on https://1.envato.market/forcaster-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.7
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Vitaliy Nemirovskiy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\ForcasterElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Repeater;
use Merkulove\Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\ForcasterElementor\Unity\Plugin as UnityPlugin;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Css_Filter;
use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Border;
use Merkulove\ForcasterElementor\Unity\Settings;

/** @noinspection PhpUnused */
/**
 * forcaster - Custom Elementor Widget.
 **/
class hourly_forecast_elementor extends Widget_Base
{

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @param array $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     * @throws \Exception
     * @throws Exception|Exception If arguments are missing when initializing a full widget instance.
     */
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        /** Register plugin styles. */
        wp_register_style(
            'mdp-forcaster-elementor',
            UnityPlugin::get_url() . 'css/forcaster-elementor' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );

        /** Register the plugin styles for the admin part. */
        wp_register_style(
            'mdp-forcaster-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );

        /** Register jquery plugin (https://jqueryui.com). */
        wp_register_script(
            'jquery-ui',
            UnityPlugin::get_url() . 'js/jquery-ui' . UnityPlugin::get_suffix() . '.js', ['jquery'],
            UnityPlugin::get_version(),
            true
        );

        add_filter( 'forecaster_country', [ $this,'filter_country' ] );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name(){ return 'mdp-forcaster-hourly-forecast-elementor'; }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title(){ return esc_html__('Hourly Forecast', 'forcaster-elementor'); }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon(){ return 'mdp-hourly-forecast-elementor-widget-icon'; }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories(){ return ['general']; }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords(){ return ['Merkulove', 'forcaster', 'weather', 'forecast', 'synoptic']; }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends(){ return ['mdp-forcaster-elementor', 'mdp-forcaster-elementor-admin']; }

    /**
     * Get script dependencies.
     * Retrieve the list of script dependencies the element requires.
     *
     * @access public
     *
     * @return array Element scripts dependencies.
     **/
    public function get_script_depends(){ return []; }

    /**
     * We get a list of available countries.
     *
     * @since 1.0.0
     * @access public
     *
     * @return array
     */
    public function get_country_options() {

        /** A variable that will store an array of countries. */
        $country_code = [
            'IR' => 'IR',
            'CY' => 'CY',
            'SO' => 'SO',
            'YE' => 'YE',
            'LY' => 'LY',
            'IQ' => 'IQ',
            'SA' => 'SA',
            'AO' => 'AO',
            'AZ' => 'AZ',
            'TZ' => 'TZ',
            'TM' => 'TM',
            'SY' => 'SY',
            'AM' => 'AM',
            'ZM' => 'ZM',
            'KE' => 'KE',
            'RW' => 'RW',
            'CD' => 'CD',
            'DJ' => 'DJ',
            'UG' => 'UG',
            'MW' => 'MW',
            'CF' => 'CF',
            'SC' => 'SC',
            'TD' => 'TD',
            'JO' => 'JO',
            'GR' => 'GR',
            'LB' => 'LB',
            'PS' => 'PS',
            'IL' => 'IL',
            'KW' => 'KW',
            'OM' => 'OM',
            'QA' => 'QA',
            'BH' => 'BH',
            'AE' => 'AE',
            'TR' => 'TR',
            'ET' => 'ET',
            'ER' => 'ER',
            'EG' => 'EG',
            'AL' => 'AL',
            'SD' => 'SD',
            'SS' => 'SS',
            'BI' => 'BI',
            'RU' => 'RU',
            'LV' => 'LV',
            'EE' => 'EE',
            'LT' => 'LT',
            'UZ' => 'UZ',
            'SE' => 'SE',
            'KZ' => 'KZ',
            'GE' => 'GE',
            'UA' => 'UA',
            'MD' => 'MD',
            'BY' => 'BY',
            'FI' => 'FI',
            'RO' => 'RO',
            'HU' => 'HU',
            'SK' => 'SK',
            'BG' => 'BG',
            'PL' => 'PL',
            'RS' => 'RS',
            'MK' => 'MK',
            'XK' => 'XK',
            'NA' => 'NA',
            'ZW' => 'ZW',
            'KM' => 'KM',
            'YT' => 'YT',
            'LS' => 'LS',
            'BW' => 'BW',
            'MU' => 'MU',
            'SZ' => 'SZ',
            'RE' => 'RE',
            'ZA' => 'ZA',
            'MZ' => 'MZ',
            'MG' => 'MG',
            'PK' => 'PK',
            'TH' => 'TH',
            'AF' => 'AF',
            'IN' => 'IN',
            'BD' => 'BD',
            'ID' => 'ID',
            'TJ' => 'TJ',
            'MY' => 'MY',
            'KG' => 'KG',
            'LK' => 'LK',
            'BT' => 'BT',
            'CN' => 'CN',
            'MV' => 'MV',
            'NP' => 'NP',
            'MM' => 'MM',
            'MN' => 'MN',
            'TF' => 'TF',
            'VN' => 'VN',
            'TL' => 'TL',
            'LA' => 'LA',
            'TW' => 'TW',
            'PH' => 'PH',
            'HK' => 'HK',
            'BN' => 'BN',
            'MO' => 'MO',
            'KH' => 'KH',
            'KR' => 'KR',
            'JP' => 'JP',
            'KP' => 'KP',
            'SG' => 'SG',
            'AU' => 'AU',
            'CX' => 'CX',
            'FM' => 'FM',
            'PG' => 'PG',
            'SB' => 'SB',
            'KI' => 'KI',
            'TV' => 'TV',
            'MH' => 'MH',
            'VU' => 'VU',
            'NC' => 'NC',
            'NF' => 'NF',
            'NZ' => 'NZ',
            'FJ' => 'FJ',
            'CM' => 'CM',
            'SN' => 'SN',
            'CG' => 'CG',
            'PT' => 'PT',
            'LR' => 'LR',
            'CI' => 'CI',
            'GH' => 'GH',
            'GQ' => 'GQ',
            'NG' => 'NG',
            'BF' => 'BF',
            'TG' => 'TG',
            'GW' => 'GW',
            'MR' => 'MR',
            'BJ' => 'BJ',
            'GA' => 'GA',
            'SL' => 'SL',
            'ST' => 'ST',
            'GI' => 'GI',
            'GM' => 'GM',
            'GN' => 'GN',
            'NE' => 'NE',
            'ML' => 'ML',
            'EH' => 'EH',
            'TN' => 'TN',
            'DZ' => 'DZ',
            'ES' => 'ES',
            'IT' => 'IT',
            'MA' => 'MA',
            'MT' => 'MT',
            'DK' => 'DK',
            'FO' => 'FO',
            'IS' => 'IS',
            'GB' => 'GB',
            'CH' => 'CH',
            'SJ' => 'SJ',
            'NL' => 'NL',
            'AT' => 'AT',
            'BE' => 'BE',
            'DE' => 'DE',
            'LU' => 'LU',
            'IE' => 'IE',
            'FR' => 'FR',
            'MC' => 'MC',
            'AD' => 'AD',
            'AX' => 'AX',
            'LI' => 'LI',
            'JE' => 'JE',
            'IM' => 'IM',
            'GG' => 'GG',
            'CZ' => 'CZ',
            'NO' => 'NO',
            'SM' => 'SM',
            'BA' => 'BA',
            'HR' => 'HR',
            'SI' => 'SI',
            'ME' => 'ME',
            'SH' => 'SH',
            'BB' => 'BB',
            'CV' => 'CV',
            'GY' => 'GY',
            'GF' => 'GF',
            'SR' => 'SR',
            'BR' => 'BR',
            'GL' => 'GL',
            'PM' => 'PM',
            'GS' => 'GS',
            'FK' => 'FK',
            'AR' => 'AR',
            'PY' => 'PY',
            'UY' => 'UY',
            'VE' => 'VE',
            'MX' => 'MX',
            'JM' => 'JM',
            'DO' => 'DO',
            'CW' => 'CW',
            'SX' => 'SX',
            'CU' => 'CU',
            'MQ' => 'MQ',
            'BS' => 'BS',
            'BM' => 'BM',
            'AI' => 'AI',
            'TT' => 'TT',
            'KN' => 'KN',
            'DM' => 'DM',
            'AG' => 'AG',
            'LC' => 'LC',
            'TC' => 'TC',
            'AW' => 'AW',
            'VG' => 'VG',
            'VC' => 'VC',
            'MS' => 'MS',
            'GP' => 'GP',
            'MF' => 'MF',
            'BL' => 'BL',
            'GD' => 'GD',
            'KY' => 'KY',
            'BZ' => 'BZ',
            'SV' => 'SV',
            'GT' => 'GT',
            'HN' => 'HN',
            'NI' => 'NI',
            'CR' => 'CR',
            'EC' => 'EC',
            'CO' => 'CO',
            'PE' => 'PE',
            'PA' => 'PA',
            'HT' => 'HT',
            'CL' => 'CL',
            'BO' => 'BO',
            'PN' => 'PN',
            'TO' => 'TO',
            'PF' => 'PF',
            'WF' => 'WF',
            'WS' => 'WS',
            'CK' => 'CK',
            'NU' => 'NU',
            'GU' => 'GU',
            'US' => 'US',
            'PR' => 'PR',
            'VI' => 'VI',
            'AS' => 'AS',
            'CA' => 'CA',
            'VA' => 'VA',
            'PW' => 'PW',
            'CC' => 'CC',
            'NR' => 'NR',
            'MP' => 'MP',
            'BQ' => 'BQ'
        ];

        /** Json with the full name of the countries. */
        $remote_json = wp_remote_get( UnityPlugin::get_url() . 'json/country.list.json' );

        if ( wp_remote_retrieve_response_code( $remote_json ) === 200 ) {

            $FullCountryName = json_decode( wp_remote_retrieve_body( $remote_json ), true, 10 );
            if ( is_array( $FullCountryName ) ) {

                /** Will save an array of countries with full name. */
                $country_name = array();

                /** Create an array of countries. */
                foreach ( $FullCountryName as $name){ $country_name[$name['Code']] = apply_filters( 'forecaster_country', $name['Name'] ); }

                /** Combine the arrays by country code into one array. */
                return array_merge( $country_code, $country_name );

            } else {

                /** Return country codes id county full list in to accessible */
                return $country_code;

            }

        }

        return $country_code;

    }

    /**
     * Returns a list of languages.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     */
    private function get_language_options() {

        /** We are connected to the file which contains the list of countries. */
        $json = file_get_contents( UnityPlugin::get_url() . 'json/language.list.json' );
        $CodeLanguage = json_decode( $json, true );

        /** A variable that will store an array of countries. */
        $result = null;

        /** Save an array of countries into a variable. */
        foreach ($CodeLanguage as $i => $iValue ) {
            $result[ $iValue['code']] = $CodeLanguage[$i]['name'];
        }

        return $result;

    }

    /**
     * Returns an array with time zones.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     */
    private function get_timezone_options() {

        /** We write an array from a file with a list of time zones into a variable. */
        return include UnityPlugin::get_path() .'array/timezones_array.php';

    }

    /**
     * Settings for slider buttons.
     *
     * @param $slug - Slug settings.
     */
    private function controls_slider_buttons( $slug = null ) {
        $slug_text = '';

        /** We check slug. */
        if( ! empty($slug) ){
            $slug_text = '-' . str_replace('_', '-', $slug );
        }

        $class_button = '{{WRAPPER}} .mdp-button-info';

        $this->add_responsive_control(
            'style_hourly_margin_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Margin', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_button => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );

        $this->add_responsive_control(
            'style_hourly_padding_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Padding', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'separator' => 'after',
                'selectors' => [
                    $class_button => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );

        $this->add_control(
            'style_hourly_icon_size_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Icon size', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    $class_button => 'font-size: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $this->add_control(
            'style_hourly_buttons_height' . $slug,
            [
                'label' => esc_html__( 'Height', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    $class_button => 'height: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $this->start_controls_tabs( 'style_hourly_tabs_style_buttons_' . $slug );

        $this->start_controls_tab(
            'style_hourly_normal_tab_style_buttons' . $slug,
                  ['label' => esc_html__( 'NORMAL', 'forcaster-elementor' )]
        );

        $this->add_control(
            'style_hourly_normal_color_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Icon color', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    $class_button => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_normal_background_style_buttons_' . $slug,
                'label' => esc_html__( 'Background type', 'forcaster-elementor' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => $class_button,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_border_normal_style_buttons_' . $slug,
                'label' =>  esc_html__( 'Border style buttons', 'forcaster-elementor' ),
                'selector' => $class_button,
            ]
        );

        $this->add_responsive_control(
            'style_hourly_border_radius_normal_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Border radius buttons', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_button => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_box_shadow_normal_style_buttons_' . $slug,
                'label' => esc_html__( 'Box Shadow', 'forcaster-elementor' ),
                'selector' => $class_button,
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'style_hourly_hover_tab_style_buttons' . $slug,
            ['label' => esc_html__( 'HOVER', 'forcaster-elementor' )]
        );

        $this->add_control(
            'style_hourly_hover_color_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Icon color', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    $class_button . ':hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_hover_background_style_buttons_' . $slug,
                'label' => esc_html__( 'Background type', 'forcaster-elementor' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => $class_button . ':hover',
            ]
        );


        $this->add_control(
            'style_hourly_hover_transition_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Hover transition', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    $class_button => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_border_hover_style_buttons_' . $slug,
                'label' =>  esc_html__( 'Border style buttons', 'forcaster-elementor' ),
                'selector' => $class_button . ':hover',
            ]
        );

        $this->add_responsive_control(
            'style_hourly_border_radius_hover_style_buttons_' . $slug,
            [
                'label' => esc_html__( 'Border radius buttons', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_button . ':hover' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_box_shadow_hover_style_buttons_' . $slug,
                'label' => esc_html__( 'Box Shadow', 'forcaster-elementor' ),
                'selector' => $class_button . ':hover',
            ]
        );


        $this->end_controls_tab();

        $this->end_controls_tabs();

    }

    /**
     * Group of fields to set the image size.
     *
     */
    private function controls_img_size() {
        /** We generate the class name for setting styles. */
        $class = '{{WRAPPER}} .mdp-hourly-img-icon';

        /** Icon Padding */
        $this->add_responsive_control(
            'style_hourly_icon_padding',
            [
                'label' => esc_html__( 'Icon Padding', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'separator' => 'before',
                'toggle' => true,
            ]
        );

        /** Icon Margin */
        $this->add_responsive_control(
            'style_hourly_icon_margin',
            [
                'label' => esc_html__( 'Icon Margin', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );


        /** Image size. */
        $this->add_responsive_control(
            'style_hourly_img_width',
            [
                'label' => esc_html__( 'Image size', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 100,
                ],
                'selectors' => [
                    $class => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        /** Icon size. */
        $this->add_responsive_control(
            'style_hourly_icon_size_image',
            [
                'label' => esc_html__( 'Icon size', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => ['px'],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-icon .mdp-forcaster-elementor-hourly-icon' => 'font-size: {{SIZE}}{{UNIT}}; height: auto;',
                    '{{WRAPPER}} .mdp-icon .mdp-forcaster-elementor-hourly-icon svg' => 'width: {{SIZE}}{{UNIT}}; height: auto;',
                ],
            ]
        );

        $this->add_responsive_control(
            'style_hourly_icon_spacing_icon',
            [
                'label' => esc_html__( 'Icon spacing', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class . ' .mdp-forcaster-elementor-hourly-icon' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                    '{{WRAPPER}} .mdp-forcaster-elementor-visual-data-icon' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};'
                ],
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'style_hourly_icon_align',
            [
                'label' => esc_html__( 'Alignment', 'forcaster-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'forcaster-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'forcaster-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'forcaster-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-icon' => 'justify-content: {{VALUE}};',
                ],
                'toggle' => true,
            ]
        );

        # Start Basic Style Tabs
        $this->start_controls_tabs( 'tabs_hourly_icon' );

        # Start Normal Tab
        $this->start_controls_tab(
            'tabs_hourly_icon_normal',
            [ 'label' => esc_html__( 'Normal', 'forcaster-elementor' ), ]
        );

        /** Icon Color */
        $this->add_control(
            'style_hourly_icon_color',
            [
                'label' => esc_html__( 'Color', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-forcaster-elementor-hourly-icon' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** CSS Filter */
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name' => 'style_hourly_css_filter',
                'selector' => $class,
            ]
        );

        /** Icon Background. */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_icon_background_box',
                'label' => esc_html__( 'Background', 'forcaster-elementor' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class,
            ]
        );

        /** Icon Border radius. */
        $this->add_responsive_control(
            'style_hourly_icon_border_radius',
            [
                'label' => esc_html__( 'Icon Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        /** Box Shadow. */
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_img_shadow',
                'label' => esc_html__( 'Box Shadow', 'forcaster-elementor' ),
                'selector' => $class,
            ]
        );

        $this->end_controls_tab();

        # Start Hover Tab
        $this->start_controls_tab(
            'tabs_hourly_icon_hover',
            [ 'label' => esc_html__( 'Hover', 'forcaster-elementor' ), ]
        );

        /** Icon Color */
        $this->add_control(
            'style_hourly_icon_color_hover',
            [
                'label' => esc_html__( 'Color', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-forcaster-elementor-hourly-icon:hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** CSS Filter */
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(),
            [
                'name' => 'style_hourly_css_filter_hover',
                'selector' => $class . ':hover',
            ]
        );

        /** Icon Background. */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_icon_background_box_hover',
                'label' => esc_html__( 'Background', 'forcaster-elementor' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class . ':hover',
            ]
        );

        /** Icon Border radius. */
        $this->add_responsive_control(
            'style_hourly_icon_border_radius_hover',
            [
                'label' => esc_html__( 'Icon Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class . ':hover' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        /** Box Shadow. */
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_img_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'forcaster-elementor' ),
                'selector' => $class . ':hover',
            ]
        );

        $this->add_control(
            'style_hourly_hover_transition_style_icon',
            [
                'label' => esc_html__( 'Hover transition', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    $class . ', {{WRAPPER}} .mdp-forcaster-elementor-hourly-icon' => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

    }

    private function controls_floating_panel_box() {
        /** We generate style class for the panel container with icons. */
        $class_box = '{{WRAPPER}} .mdp-sticky-' . 'hourly';

        $this->start_controls_section(
            'style_section_hourly_float_box',
            [
                'label' => esc_html__( 'Floating Panel Box', 'forcaster-elementor' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'forcaster_floating_panel' => 'yes'
                ]
            ]
        );

        # Margin
        $this->add_responsive_control(
            'style_hourly_margin_panel_box',
            [
                'label' => esc_html__( 'Margin', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_box => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Padding
        $this->add_responsive_control(
            'style_hourly_padding_panel_box',
            [
                'label' => esc_html__( 'Padding', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_box => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->start_controls_tabs( 'style_hourly_tabs_style_floating_panel_box' );

        $this->start_controls_tab(
            'style_hourly_normal_tab_style_floating_panel_box',
            ['label' => esc_html__( 'NORMAL', 'forcaster-elementor' )]
        );

        /** Background */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_normal_background_box',
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class_box,
            ]
        );

        # Border style
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_normal_box_border_style_panel_box',
                'label' =>  esc_html__( 'Border', 'forcaster-elementor' ),
                'selector' => $class_box,
                'separator' => 'before'
            ]
        );

        # Border radius
        $this->add_responsive_control(
            'style_hourly_normal_box_border_radius_panel_box',
            [
                'label' => esc_html__( 'Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_box => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_normal_box_shadow_floating_panel_box',
                'selector' => $class_box,
                'separator' => 'before',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'style_hourly_hover_tab_style_floating_panel_box',
            [ 'label' => esc_html__( 'HOVER', 'forcaster-elementor' ) ]
        );

        /** Background */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_hover_background_panel_box',
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class_box . ':hover',
            ]
        );

        $this->add_control(
            'style_hourly_hover_transition_panel_box',
            [
                'label' => esc_html__( 'Hover transition duration', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    $class_box => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        # Border style
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_hover_box_border_style_panel_box',
                'label' =>  esc_html__( 'Border', 'forcaster-elementor' ),
                'selector' => $class_box . ':hover',
                'separator' => 'before'
            ]
        );

        # Border radius
        $this->add_responsive_control(
            'style_hourly_hover_box_border_radius_panel_box',
            [
                'label' => esc_html__( 'Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_box . ':hover' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_hover_box_shadow_floating_panel_box',
                'selector' => $class_box . ':hover',
                'separator' => 'before',
            ]
        );

        $this->end_controls_tabs();

        # End section
        $this->end_controls_section();
    }

    /**
     *  A group of fields for customizing a panel with icons.
     *
     * @since 1.0.0
     * @access public
     */
    private function controls_floating_panel_item() {
        /** We generate style class for the panel container with icons. */
        $class_box = '{{WRAPPER}} .mdp-sticky-' . 'hourly';

        /** We generate style class for child elements. */
        $class = '{{WRAPPER}} .mdp-sticky-hourly > div';

        /** We generate style class for text. */
        $text_class = '{{WRAPPER}} .mdp-floating-text';

        /** We generate style class for icons. */
        $class_floating = '{{WRAPPER}} .mdp-hourly-floating-icon';

        $this->start_controls_section(
            'style_section_hourly_float',
            [
                'label' => esc_html__( 'Floating Panel Item', 'forcaster-elementor' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'forcaster_floating_panel' => 'yes'
                ]
            ]
        );

        # Margin
        $this->add_responsive_control(
            'style_hourly_margin_panel',
            [
                'label' => esc_html__( 'Margin', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Padding
        $this->add_responsive_control(
            'style_hourly_padding_panel',
            [
                'label' => esc_html__( 'Padding', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'separator' => 'after',
                'toggle' => true,
            ]
        );

        # Size icon
        $this->add_responsive_control(
            'style_hourly_size_icon',
            [
                'label' => esc_html__( 'Size icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'default' => [
                    'unit' => 'px',
                    'size' => 16,
                ],
                'selectors' => [
                    $class_floating => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'style_hourly_space_between_items',
            [
                'label' => esc_html__( 'Space between items', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'px' => [
                    'min' => 0,
                    'max' => 500,
                    'step' => 1,
                ],
                'selectors' => [
                    $class => 'margin-bottom: {{SIZE}}{{UNIT}};',
                    $class . ':last-child' => 'margin-bottom: 0;',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'style_hourly_typography_floating_panel',
                'scheme' => Typography::TYPOGRAPHY_3,
                'selector' => $text_class,
            ]
        );

        $this->start_controls_tabs( 'style_hourly_tabs_style_floating_panel' );

        $this->start_controls_tab(
            'style_hourly_normal_tab_style_floating_panel',
            ['label' => esc_html__( 'NORMAL', 'forcaster-elementor' )]
        );

        # Color floating icon
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_control(
            'style_hourly_normal_color_floating_icon',
            [
                'label' => esc_html__( 'Color icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    $class_floating . ', ' . $text_class => 'color: {{VALUE}}',
                ],
            ]
        );

        /** Text Shadow */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'style_hourly_normal_shadow',
                'selector' => $class,
            ]
        );

        /** Background */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_normal_background',
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class,
            ]
        );

        # Border style
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_normal_box_border_style',
                'label' =>  esc_html__( 'Border', 'forcaster-elementor' ),
                'selector' => $class,
                'separator' => 'before'
            ]
        );

        # Border radius
        $this->add_responsive_control(
            'style_hourly_normal_box_border_radius',
            [
                'label' => esc_html__( 'Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_normal_box_shadow_floating_panel',
                'selector' => $class,
                'separator' => 'before',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'style_hourly_hover_tab_style_floating_panel',
            [ 'label' => esc_html__( 'HOVER', 'forcaster-elementor' ) ]
        );

        # Color floating icon
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_control(
            'style_hourly_hover_color_floating_icon',
            [
                'label' => esc_html__( 'Color', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    $class . ':hover .mdp-hourly-floating-icon, {{WRAPPER}} .mdp-floating-text:hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** Text Shadow */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'style_hourly_hover_shadow',
                'selector' => $class . ':hover',
            ]
        );

        /** Background */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_hover_background',
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class . ':hover',
            ]
        );

        $this->add_control(
            'style_hourly_hover_transition',
            [
                'label' => esc_html__( 'Hover transition duration', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    $class . ', ' . $class_floating . ', ' . $text_class => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        # Border style
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_hover_box_border_style',
                'label' =>  esc_html__( 'Border', 'forcaster-elementor' ),
                'selector' => $class_box . ':hover',
                'separator' => 'before'
            ]
        );

        # Border radius
        $this->add_responsive_control(
            'style_hourly_hover_box_border_radius',
            [
                'label' => esc_html__( 'Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class_box . ':hover' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_hover_box_shadow_floating_panel',
                'selector' => $class . ':hover',
                'separator' => 'before',
            ]
        );

        $this->end_controls_tabs();

        # End section
        $this->end_controls_section();

    }

    /**
     * Section layout. Layout order settings.
     *
     * @since 1.0.0
     * @access private
     */
    private function section_layout() {

        /** Start section layout. */
        $this->start_controls_section( 'section_layout',
            [ 'label' => esc_html__( 'Layout', 'forcaster-elementor' ) ] );

        $this->add_control(
            'forcaster_auto_location',
            [
                'label' => esc_html__( 'Auto location', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
            ]
        );

        /** Country. */
        $this->add_control(
            'location_country',
            [
                'label' => esc_html__( 'Country', 'forcaster-elementor' ),
                'type' => Controls_Manager::SELECT2,
                'multiple' => false,
                'options' => $this->get_country_options(),
                'default' => 'UA',
                'label_block' => true,
                'condition' => [ 'forcaster_auto_location!' => 'yes' ]
            ]
        );

        /** City. */
        $this->add_control(
            'location_city',
            [
                'label' => esc_html__( 'City', 'forcaster-elementor' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'default' => esc_html__( 'Mykolayiv', 'forcaster-elementor' ),
                'placeholder' => esc_html__( 'City', 'forcaster-elementor' ),
                'separator' => 'after',
                'condition' => [ 'forcaster_auto_location!' => 'yes' ]
            ]
        );

        /** Language country. */
        $this->add_control(
            'language_country',
            [
                'label' => esc_html__( 'Language', 'forcaster-elementor' ),
                'type' => Controls_Manager::SELECT2,
                'multiple' => false,
                'options' => $this->get_language_options(),
                'default' => 'uk',
                'label_block' => true
            ]
        );

        /** Timezone. */
        $this->add_control(
				'timezone_country',
				[
					'label' => esc_html__( 'Timezone', 'forcaster-elementor' ),
					'type' => Controls_Manager::SELECT2,
					'multiple' => false,
					'options' => $this->get_timezone_options(),
					'default' => 'Europe/Kiev',
					'label_block' => true
				]
			);

        /** Units. */
        $this->add_control(
            'location_units',
            [
                'label' => esc_html__( 'Units', 'forcaster-elementor' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'standard' => esc_html__( 'Kelvin', 'forcaster-elementor' ),
                    'metric' => esc_html__( 'Celsius', 'forcaster-elementor' ),
                    'imperial' => esc_html__( 'Fahrenheit', 'forcaster-elementor' ),
                ],
                'default' => 'metric',
            ]
        );

        /** Round. */
        $this->add_control(
            'forcaster_round',
            [
                'label' => esc_html__( 'Round', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes'
            ]
        );

        $this->add_control(
            'forcaster_floating_panel',
            [
                'label' => esc_html__( 'Floating panel', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes'
            ]
        );

        $this->add_control(
            'forcaster_hide_buttons',
            [
                'label' => esc_html__( 'Hide navigation buttons', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no'
            ]
        );

        /** End section layout. */
        $this->end_controls_section();

    }

    /**
     * Single content entity controls
     *
     * @param string $icon - Entity Icon
     * @param string $slug - Entity Slug
     * @param string $heading - Entity Heading
     *
     * @since 1.0.0
     * @access public
     */
    private function single_content( $repeater, $slug, $heading, $icon, $data_type ) {
        # Icon Switcher
        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_i',
            [
                'label' => $heading . esc_html__( ' Icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type,
                ]
            ]
        );

        # Icon
        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_icon',
            [
                'type' => Controls_Manager::ICONS,
                'fa4compatibility' => 'icon',
                'default' => [
                    'value' => 'fas ' . $icon,
                    'library' => 'fa-solid',
                ],
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type,
                    'forcaster_hourly_' . $slug . '_i' => 'yes'
                ]
            ]
        );

        if ( $slug === 'visual_data' ) {
            $repeater->add_control(
                'forcaster_hourly_visual_data_icon_style',
                [
                    'type' => Controls_Manager::SELECT,
                    'default' => 'real/',
                    'label' => esc_html__( 'Icon style', 'forcaster-elementor' ),
                    'options' => [
                        'real/'     => esc_html__( 'Realistic', 'forcaster-elementor' ),
                        'gradient/' => esc_html__( 'Gradient', 'forcaster-elementor' ),
                        'outline/'  => esc_html__( 'Outline', 'forcaster-elementor' ),
                    ],
                    'condition' => [
                        'forcaster_weather_hourly_info_type' => 'icon',
                    ]
                ]
            );
        }

        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_icon_position',
            [
                'type' => Controls_Manager::SELECT,
                'default' => 'left',
                'label' => esc_html__( 'Icon position', 'forcaster-elementor' ),
                'options' => [
                    'left'     => esc_html__( 'Left', 'forcaster-elementor' ),
                    'above' => esc_html__( 'Top', 'forcaster-elementor' ),
                    'right' => esc_html__( 'Right', 'forcaster-elementor' ),
                    'under' => esc_html__( 'Bottom', 'forcaster-elementor' ),
                ],
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type,
                    'forcaster_hourly_' . $slug . '_i' => 'yes'
                ]
            ]
        );

        # Caption
        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_caption',
            [
                'label' => $heading . esc_html__( ' Caption', 'forcaster-elementor' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => $heading . ':',
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type
                ]
            ]
        );

        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_caption_position',
            [
                'type' => Controls_Manager::SELECT,
                'default' => 'left',
                'label' => esc_html__( 'Caption position', 'forcaster-elementor' ),
                'options' => [
                    'left'     => esc_html__( 'Left', 'forcaster-elementor' ),
                    'above' => esc_html__( 'Top', 'forcaster-elementor' ),
                    'right' => esc_html__( 'Right', 'forcaster-elementor' ),
                    'under' => esc_html__( 'Bottom', 'forcaster-elementor' ),
                ],
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type,
                ]
            ]
        );

        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_display_in_panel',
            [
                'type' => Controls_Manager::SELECT,
                'default' => 'icon',
                'label' => esc_html__( 'Show in floating panel', 'forcaster-elementor' ),
                'options' => [
                    'icon'     => esc_html__( 'Icon', 'forcaster-elementor' ),
                    'text' => esc_html__( 'Text', 'forcaster-elementor' ),
                ],
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type,
                ]
            ]
        );

        # Caption
        $repeater->add_control(
            'forcaster_hourly_' . $slug . '_panel_text',
            [
                'label' => esc_html__( 'Floating panel text', 'forcaster-elementor' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => $heading . ':',
                'condition' => [
                    'forcaster_weather_hourly_info_type' => $data_type,
                    'forcaster_hourly_' . $slug . '_display_in_panel' => 'text'
                ]
            ]
        );

    }

    /**
     * Section hourly. Settings for displaying weather with an interval of three hours.
     *
     * @since 1.0.0
     * @access private
     */
    private function section_hourly() {

        /** Section start */
        $this->start_controls_section( 'section_hourly', [
            'label' => esc_html__( 'Hourly', 'forcaster-elementor' ),
        ] );

        $this->add_control(
            'hourly_time_format',
            [
                'label' => esc_html__( 'Time format', 'forcaster-elementor' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h:i A' => esc_html__( 'AM/PM', 'forcaster-elementor' ),
                    'H:i' => esc_html__( '24 Hour', 'forcaster-elementor' ),
                    'wp-format' => esc_html__( 'Wordpress format', 'forcaster-elementor' ),
                ],
                'default' => 'wp-format',
            ]
        );

        /** Quantity items */
        $this->add_control(
            'style_hourly_quantity_items',
            [
                'label' => esc_html__( 'Quantity items', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'size_units' => ['px'],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 40,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 10,
                ],
            ]
        );

        /** Scrolling items */
        $this->add_responsive_control(
            'style_hourly_scrolling_items',
            [
                'label' => esc_html__( 'Scrolling items', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'default' => [
                    'unit' => 'px',
                    'size' => 3,
                ],

            ]
        );


        $this->add_responsive_control(
            'style_weather_align_location-box',
            [
                'label' => esc_html__( 'Location', 'forcaster-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'forcaster-elementor' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'forcaster-elementor' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'forcaster-elementor' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'space-around' => [
                        'title' => esc_html__( 'Space around', 'forcaster-elementor' ),
                        'icon' => 'fa fa-compress-arrows-alt',
                    ],
                    'space-between' => [
                        'title' => esc_html__( 'Space between', 'forcaster-elementor' ),
                        'icon' => 'fa fa-expand-arrows-alt',
                    ],
                ],
                'conditions' => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name' => 'forcaster_weather_country',
                            'operator' => '==',
                            'value' => 'yes'
                        ],
                        [
                            'name' => 'forcaster_weather_location',
                            'operator' => '==',
                            'value' => 'yes'
                        ],
                        [
                            'name' => 'forcaster_weather_population',
                            'operator' => '==',
                            'value' => 'yes'
                        ],
                    ]
                ],
                'default' => 'space-between',
                'selectors' => [
                    '{{WRAPPER}} .mdp-forcaster-weather-box' => 'justify-content: {{VALUE}};',
                ],
                'toggle' => true,
            ]
        );

        /** City. */
        $this->add_control(
            'forcaster_weather_location',
            [
                'label' => esc_html__( 'City', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        /** City caption */
        $this->add_control(
            'forcaster_weather_location_caption',
            [
                'label' => esc_html__( 'City Caption', 'forcaster-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Location:', 'forcaster-elementor' ),
                'placeholder' => esc_html__( 'City caption', 'forcaster-elementor' ),
                'condition' => ['forcaster_weather_location' => 'yes']
            ]
        );

        /** City Icon. */
        $this->add_control(
            'forcaster_weather_location_icon',
            [
                'label' => esc_html__( 'City Icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::ICONS,
                'fa4compatibility' => 'icon',
                'condition' => ['forcaster_weather_location' => 'yes']
            ]
        );

        /** Country */
        $this->add_control(
            'forcaster_weather_country',
            [
                'label' => esc_html__( 'Country', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes'
            ]
        );

        /** Country caption */
        $this->add_control(
            'forcaster_weather_country_caption',
            [
                'label' => esc_html__( 'Country Caption', 'forcaster-elementor' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Country caption', 'forcaster-elementor' ),
                'condition' => ['forcaster_weather_country' => 'yes']
            ]
        );

        /** Country Icon. */
        $this->add_control(
            'forcaster_weather_country_icon',
            [
                'label' => esc_html__( 'Country Icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::ICONS,
                'fa4compatibility' => 'icon',
                'condition' => ['forcaster_weather_country' => 'yes']
            ]
        );


        /** Population. */
        $this->add_control(
            'forcaster_weather_population',
            [
                'label' => esc_html__( 'Population', 'forcaster-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes'
            ]
        );

        /** Population caption */
        $this->add_control(
            'forcaster_weather_population_caption',
            [
                'label' => esc_html__( 'Population Caption', 'forcaster-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Population:', 'forcaster-elementor' ),
                'placeholder' => esc_html__( 'Population caption', 'forcaster-elementor' ),
                'condition' => ['forcaster_weather_population' => 'yes']
            ]
        );

        /** Population Icon. */
        $this->add_control(
            'forcaster_weather_population_icon',
            [
                'label' => esc_html__( 'Population Icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::ICONS,
                'fa4compatibility' => 'icon',
                'condition' => ['forcaster_weather_population' => 'yes']
            ]
        );


        $repeater = new Repeater();

        $repeater->add_control(
            'forcaster_weather_hourly_info_type',
            [
                'label' => esc_html__( 'Weather info', 'forcaster-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'date',
                'options' => [
                    'date' => esc_html__( 'Date', 'forcaster-elementor' ),
                    'temp'  => esc_html__( 'Temperature', 'forcaster-elementor' ),
                    'feels-like'  => esc_html__( 'Feels like', 'forcaster-elementor' ),
                    'icon'  => esc_html__( 'Icon', 'forcaster-elementor' ),
                    'description'  => esc_html__( 'Description', 'forcaster-elementor' ),
                    'pressure'  => esc_html__( 'Pressure', 'forcaster-elementor' ),
                    'humidity'  => esc_html__( 'Humidity', 'forcaster-elementor' ),
                    'cloudy'  => esc_html__( 'Cloudy', 'forcaster-elementor' ),
                    'wind-speed'  => esc_html__( 'Wind speed', 'forcaster-elementor' ),
                    'wind-direction'  => esc_html__( 'Wind direction', 'forcaster-elementor' ),
                ],
            ]
        );

        /** Date and Time */
        $this->single_content(
            $repeater,
            'date',
            esc_html__('Date', 'forcaster-elementor'),
            'fa-calendar',
            'date'
        );

        /** Temperature */
        $this->single_content(
            $repeater,
            'temp',
            esc_html__('Temperature', 'forcaster-elementor'),
            'fa-temperature-low',
            'temp'
        );

        /** Feels Like */
        $this->single_content(
            $repeater,
            'feels_like',
            esc_html__('Feels like', 'forcaster-elementor'),
            'fa-hand-holding-heart',
            'feels-like'
        );

        /** Visual Data Icon */
        $this->single_content(
            $repeater,
            'visual_data',
            esc_html__( 'Visual data', 'forcaster-elementor' ),
            'fas fa-eye',
            'icon'
        );

        /** Description */
        $this->single_content(
            $repeater,
            'description',
            esc_html__('Description', 'forcaster-elementor'),
            'fa-bullhorn',
            'description'
        );

        /** Pressure */
        $this->single_content(
            $repeater,
            'pressure',
            esc_html__('Pressure', 'forcaster-elementor'),
            'fa-tachometer-alt',
            'pressure'
        );

        /** Humidity */
        $this->single_content(
            $repeater,
            'humidity',
            esc_html__('Humidity', 'forcaster-elementor'),
            'fa-tint',
            'humidity'
        );

        /** Cloudy */
        $this->single_content(
            $repeater,
            'clouds_all',
            esc_html__('Cloudy', 'forcaster-elementor'),
            'fa-cloud',
            'cloudy'
        );

        /** Wind Speed */
        $this->single_content(
            $repeater,
            'wind_speed',
            esc_html__('Wind Speed', 'forcaster-elementor'),
            'fa-wind',
            'wind-speed'
        );

        /** Wind Direction */
        $this->single_content(
            $repeater,
            'wind_deg',
            esc_html__('Wind Direction', 'forcaster-elementor'),
            'fa-arrows-alt',
            'wind-direction'
        );

        $this->add_control(
            'weather_hourly_data',
            [
                'label' => esc_html__( 'Weather data', 'forcaster-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'forcaster_weather_hourly_info_type' => 'date',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'icon',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'temp',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'feels-like',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'cloudy',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'pressure',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'wind-speed',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'wind-direction',
                    ],
                    [
                        'forcaster_weather_hourly_info_type' => 'description',
                    ],
                ],
                'title_field' => '{{{ forcaster_weather_hourly_info_type }}}',
            ]
        );

        $this->add_control(
            'weather_hourly_previous_btn_icon',
            [
                'label' => esc_html__( 'Previous button icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-left',
                    'library' => 'solid',
                ],
            ]
        );

        $this->add_control(
            'weather_hourly_next_btn_icon',
            [
                'label' => esc_html__( 'Next button icon', 'forcaster-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-right',
                    'library' => 'solid',
                ],
            ]
        );

        /** Section End */
        $this->end_controls_section();

    }

    /**
     * Section hourly. Settings for displaying weather with an interval of three hours.
     *
     * @since 1.0.0
     * @access private
     */
    private function section_hourly_buttons() {

        /** Section start */
        $this->start_controls_section( 'section_hourly_buttons', [
            'label' => esc_html__( 'Slide buttons', 'forcaster-elementor' ),
        ] );

        $this->add_responsive_control(
            'style_hourly_buttons_left_offset_x',
            [
                'label' => esc_html__( 'Left button offset x', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-button-left-hourly' => 'left: {{SIZE}}{{UNIT}} !important',
                ],
            ]
        );

        $this->add_responsive_control(
            'style_hourly_buttons_right_offset_x',
            [
                'label' => esc_html__( 'Right button offset x', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-button-right-hourly' => 'left: {{SIZE}}{{UNIT}}',
                ],
            ]
        );


        $this->add_responsive_control(
            'style_hourly_buttons_left_offset_y',
            [
                'label' => esc_html__( 'Left button offset y', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-button-left-hourly' => 'top: {{SIZE}}{{UNIT}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'style_hourly_buttons_right_offset_y',
            [
                'label' => esc_html__( 'Right button offset y', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-button-right-hourly' => 'top: {{SIZE}}{{UNIT}}',
                ],
            ]
        );


        $this->end_controls_section();
    }


    /**
     * Create a section for forcaster data entity in the Hourly layout
     *
     * @param string $slug Entity type
     * @param string $heading Section Heading
     *
     * @since 1.0.0
     * @access private
     */
    private function style_section( $slug, $heading ) {
        # Start section
        $this->start_controls_section(
            'style_section_hourly_' . $slug,
            [
                'label' => $heading,
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        /** Entity box controls */
        $this->entity_style($slug);

        # End section
        $this->end_controls_section();

    }

    /**
     * Basic box controls suitable for normal and hover styles
     *
     * @param string $slug - Entity unique slug
     * @param string $suffix - :attribute
     *
     * @since 1.0.0
     * @access private
     */
    private function box_style( $slug, $suffix ) {
        $class = '{{WRAPPER}} .mdp-hourly-'. str_replace('_', '-', $slug ) . $suffix;

        /** Color */
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_control(
            'style_hourly_color_' . $slug . $suffix,
            [
                'label' => esc_html__( 'Color', 'forcaster-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    $class => 'color: {{VALUE}}',
                ],
            ]
        );

        if ( $slug !== 'caption' ) {
            /** Icon Color */
            /** @noinspection PhpUndefinedClassInspection */
            $this->add_control(
                'style_hourly_icon_color_' . $slug . $suffix,
                [
                    'label' => esc_html__( 'Icon color', 'forcaster-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        $class . ' .mdp-forcaster-elementor-hourly-icon' => 'color: {{VALUE}}',
                    ],
                ]
            );
        }

        if ( $slug !== 'caption' ) {
            $this->add_responsive_control(
                'style_hourly_icon_spacing_'. $slug . $suffix,
                [
                    'label' => esc_html__( 'Icon spacing', 'forcaster-elementor' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%', 'em' ],
                    'selectors' => [
                        $class . ' .mdp-forcaster-elementor-hourly-icon' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                         '{{WRAPPER}} .mdp-sticky-hourly .mdp-forcaster-elementor-'. $slug .'-icon' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};'
                    ],
                    'toggle' => true,
                ]
            );
        }

        /** Text Shadow */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'style_hourly_shadow_' . $slug . str_replace( ':', '-', $suffix ),
                'selector' => $class . $suffix,
            ]
        );

        /** Background */
        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'style_hourly_text_background_box_'. $slug . str_replace( ':', '-', $suffix ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => $class . $suffix,
            ]
        );

        /** Border style. */
        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'style_hourly_box_border_style_'. $slug . str_replace( ':', '-', $suffix ),
                'selector' => $class . $suffix,
                'separator' => 'before',
            ]
        );

        /** Border radius. */
        $this->add_responsive_control(
            'style_hourly_box_border_radius_'. $slug . $suffix,
            [
                'label' => esc_html__( 'Border radius', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class . $suffix => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        /** Box Shadow */
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'style_hourly_box_shadow_'. $slug . str_replace( ':', '-', $suffix ),
                'selector' => $class . $suffix,
                'separator' => 'before',
            ]
        );

    }

    /**
     * Single entity box style
     *
     * @param $slug
     * @since 1.0.0
     * @access private
     */
    private function entity_style( $slug ) {
        /** Prepare variables */
        $class = '{{WRAPPER}} .mdp-hourly-'. str_replace( '_', '-', $slug );

        # Margin
        $this->add_responsive_control(
            'style_hourly_margin_'. $slug,
            [
                'label' => esc_html__( 'Margin', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        # Padding
        $this->add_responsive_control(
            'style_hourly_padding_'. $slug,
            [
                'label' => esc_html__( 'Padding', 'forcaster-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    $class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'toggle' => true,
            ]
        );

        if ( $slug !== 'caption' ) {
            /** Icon size. */
            $this->add_responsive_control(
                'style_daily_icon_size_' . $slug,
                [
                    'label' => esc_html__( 'Icon size', 'forcaster-elementor' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => ['px'],
                    'range' => [
                        'px' => [
                            'min' => 0,
                            'max' => 100,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 20,
                    ],
                    'selectors' => [
                        $class . ' .mdp-forcaster-elementor-hourly-icon' => 'font-size: {{SIZE}}{{UNIT}}; height: auto;',
                        $class . ' .mdp-forcaster-elementor-hourly-icon svg' => 'width: {{SIZE}}{{UNIT}}; height: auto;',
                    ],
                ]
            );
        }

        /** Typography */
        /** @noinspection PhpUndefinedClassInspection */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'style_hourly_typography_' . $slug,
                'scheme' => Typography::TYPOGRAPHY_3,
                'selector' => $class,
            ]
        );

        if ( $slug !== 'caption' && $slug !== 'country' && $slug !== 'population' && $slug !== 'location' ) {

            $this->add_responsive_control(
                'style_hourly_align_' . $slug,
                [
                    'label' => esc_html__( 'Text align', 'forcaster-elementor' ),
                    'type' => Controls_Manager::CHOOSE,
                    'options' => [
                        'flex-start' => [
                            'title' => esc_html__( 'Left', 'forcaster-elementor' ),
                            'icon' => 'fa fa-align-left',
                        ],
                        'center' => [
                            'title' => esc_html__( 'Center', 'forcaster-elementor' ),
                            'icon' => 'fa fa-align-center',
                        ],
                        'flex-end' => [
                            'title' => esc_html__( 'Right', 'forcaster-elementor' ),
                            'icon' => 'fa fa-align-right',
                        ],
                    ],
                    'selectors' => [
                        $class => 'justify-content: {{VALUE}};',
                    ],
                    'toggle' => true,
                ]
            );

        }

        # Start Basic Style Tabs
        $this->start_controls_tabs( 'tabs_hourly_' . $slug );

        # Start Normal Tab
        $this->start_controls_tab(
            'tabs_hourly_' . $slug . '_normal',
            [ 'label' => esc_html__( 'Normal', 'forcaster-elementor' ), ]
        );

        /** Normal Styles */
        $this->box_style($slug, '');

        # End Normal Tab
        $this->end_controls_tab();

        # Start Hover Tab
        $this->start_controls_tab(
            'tabs_hourly_' . $slug . '_hover',
            [ 'label' => esc_html__( 'Hover', 'forcaster-elementor' ), ]
        );

        /** Hover Styles */
        $this->box_style($slug, ':hover');

        $this->add_control(
            'style_hourly_hover_transition_style_' . $slug,
            [
                'label' => esc_html__( 'Hover transition', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    $class => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                    $class . ' .mdp-forcaster-elementor-hourly-icon' => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        # End Hover Tab
        $this->end_controls_tab();

        # End Basic Style Tabs
        $this->end_controls_tabs();

    }

    /**
     * Style settings for captions.
     *
     * @since 1.0.0
     * @access public
     */
    private function style_caption() {
        $this->start_controls_section( 'style_hourly_caption',
            [
                'label' => esc_html__( 'Caption', 'forcaster-elementor' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->entity_style( 'caption' );

        $this->end_controls_section();
    }

    /**
     * Style settings for hourly weather.
     *
     * @since 1.0.0
     * @access private
     */
    private function style_hourly() {

        # Start section
        $this->start_controls_section(
            'style_section_hourly',
            [
                'label' => esc_html__( 'Hourly', 'forcaster-elementor' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_responsive_control(
            'hourly_column_width',
            [
                'label' => esc_html__( 'Column width', 'forcaster-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 200,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-row-hourly' => 'min-width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();

        /** Hourly Styles -> Country */
        $this->style_section( 'country', 'Country' );

        /** Hourly Styles -> Population */
        $this->style_section( 'population', 'Population' );

        /** Hourly Styles -> City */
        $this->style_section( 'location', 'City' );

        /** Hourly Styles -> Day */
        $this->style_section( 'date', 'Day and Time' );

        /** Hourly Styles -> Icon */
        $this->icon_styles();

        /** Hourly Styles -> Description */
        $this->style_section( 'description', 'Description' );

        /** Hourly Styles -> Temperature */
        $this->style_section( 'temp', 'Temp' );

        /** Hourly Styles -> Feels like */
        $this->style_section( 'feels_like', 'Feels like' );

        /** Hourly Styles -> Pressure */
        $this->style_section( 'pressure', 'Pressure' );

        /** Hourly Styles -> Humidity */
        $this->style_section( 'humidity', 'Humidity' );

        /** Hourly Styles -> Cloudy */
        $this->style_section( 'clouds_all', 'Cloudy' );

        /** Hourly Styles -> Wind Speed Section */
        $this->style_section( 'wind_speed', 'Wind speed' );

        /** Hourly Styles -> Wind Direction Section */
        $this->style_section( 'wind_deg', 'Wind direction' );

        # Start section
        $this->start_controls_section(
            'style_section_hourly_slide',
            [
                'label' => esc_html__( 'Slide buttons', 'forcaster-elementor' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        /** Slider buttons. */
        $this->controls_slider_buttons('');

        # End section
        $this->end_controls_section();

        /** Hourly Styles -> Floating Panel Box */
        $this->controls_floating_panel_box();

        /** Hourly Styles -> Floating Panel Item */
        $this->controls_floating_panel_item();

        /** Caption. */
        $this->style_caption();

    }


    /**
     * Add the icon styles controls.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function icon_styles() {
        $this->start_controls_section(
            'style_section_hourly_icon',
            [
                'label' => esc_html__( 'Icon', 'forcaster-elementor' ),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->controls_img_size();

        $this->end_controls_section();

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /** Layout order settings. */
        $this->section_layout();

        /** Settings for displaying weather with an interval of three hours. */
        $this->section_hourly();

        /** Settings for buttons. */
        $this->section_hourly_buttons();

        /** Style Hourly. */
        $this->style_hourly();

    }

    /**
     * We return the current system to the unit of measure.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     */
    private function weather_units() {

        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        $units = array();

        switch ( $settings['location_units'] ) {
            case 'standard':
                $units = array( 'temp' => 'K',
                    'humidity' => '%',
                    'pressure' => 'hPa',
                    'wind_speed' => 'm/s',
                    'wind_deg' => 'degrees',
                    'clouds_all' => '%',
                    'feels_like' => 'K');
                break;
            case 'metric':
                $units = array( 'temp' => '&#176;C',
                    'humidity' => '%',
                    'pressure' => 'hPa',
                    'wind_speed' => 'm/s',
                    'wind_deg' => 'deg',
                    'clouds_all' => '%',
                    'feels_like' => '&#176;C');
                break;
            case 'imperial':
                $units = array( 'temp' => '&#176;F',
                    'humidity' => '%',
                    'pressure' => 'hPa',
                    'wind_speed' => 'mil/h',
                    'wind_deg' => 'deg',
                    'clouds_all' => '%',
                    'feels_like' => '&#176;F');
                break;
        }

        return $units;

    }

    /**
     * Based on the current local time, we determine the nearest three-hour weather forecast.
     *
     * @since 1.0.0
     * @access private
     *
     * @return string
     */
    private function weather_time(){

        /** Current local time. */
        $t = date('H:i');

        $time = '';

        if( $t > '21:00' && $t >= '00:00' ){
            $time = ' 21:00:00';
        }

        if( $t > '18:00' && $t <= '21:00' ){
            $time = ' 18:00:00';
        }

        if( $t > '15:00' && $t <= '18:00' ){
            $time = ' 15:00:00';
        }

        if( $t > '12:00' && $t <= '15:00' ){
            $time = ' 12:00:00';
        }

        if( $t > '09:00' && $t <= '12:00' ){
            $time = ' 09:00:00';
        }

        if( $t > '06:00' && $t <= '09:00' ){
            $time = ' 06:00:00';
        }

        if( $t > '03:00' && $t <= '06:00' ){
            $time = ' 03:00:00';
        }

        if( $t > '00:00' && $t <= '03:00' ){
            $time = ' 00:00:00';
        }

        return $time;

    }

    /**
     * Return html markup for icon by value.
     *
     * @param $icon
     * @param string $title - Tile for icon. Optional.
     *
     * @return string
     */
    private function get_floating_icon( $icon, $title = null ) {
        $result = '';

        /** If we have value for the icon, prepare the markup for it. */
        if ( ! empty( $icon ) ) {
            $result_icon = $icon['library'] === 'svg' ?
                Icons_Manager::render_uploaded_svg_icon( $icon['value'] ) :
                '<i class="mdp-hourly-floating-icon '.esc_attr( $icon['value'] ).'"></i>';

            $result = sprintf(
                '<div class="mdp-hourly-floating-icon" title="%s">%s</div>',
                esc_attr__( $title ),
                $result_icon
            );
        }

        return $result;

    }

    /**
     * Return html markup for weather info icon by value.
     *
     * @param $icon
     * @return string
     */
    private function get_info_icon( $icon ) {
        $result = '';

        if ( !empty( $icon ) ) {
            $result_icon = $icon['library'] === 'svg' ?
                Icons_Manager::render_uploaded_svg_icon( $icon['value'] ) :
                '<i class="'.esc_attr( $icon['value'] ).'"></i>';

            $result = sprintf(
                '<span class="mdp-forcaster-elementor-hourly-icon">
                            %s
                        </span>',
                $result_icon
            );
        }

        return $result;

    }

    /**
     * Returns a panel with icons.
     *
     * @param $settings - Array with settings.
     * @param $class_box_icon - The class of styles for the panel with icons.
     *
     * @return string
     */
    private function floating_panel( $settings, $class_box_icon ) {
        /** Prefix for icons in settings. */
        $p = 'forcaster_hourly';

        /** Array with all icons. */
        $output = [];

        $weather_info = $settings['weather_hourly_data'];

        foreach ( $weather_info as $weather_datum ) {
            switch ( $weather_datum['forcaster_weather_hourly_info_type'] ) {
                case 'date':
                    /** Date Icon. */
                    $output[] = $weather_datum[$p . '_date_display_in_panel'] === 'icon' ?
                        sprintf( '<div class="mdp-forcaster-elementor-date-icon">%s</div>', $this->get_floating_icon(
                            $weather_datum[$p . '_date_icon'],
                            'Date'
                        ) )  : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_date_panel_text'] ) );
                    break;
                case 'icon':
                    /** Visual Data Icon. */
                    $output[] = $weather_datum[$p . '_visual_data_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-visual-data-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_visual_data_icon'],
                                'Icon'
                            )
                        )  : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_visual_data_panel_text'] ) );
                    break;
                case 'description':
                    /** Description icon. */
                    $output[] = $weather_datum[$p . '_description_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-description-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_description_icon'],
                                'Description'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_description_panel_text'] ) );
                    break;
                case 'temp':
                    /** Temp icon. */
                    $output[] = $weather_datum[$p . '_temp_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-temp-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_temp_icon'],
                                'Temp'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_temp_panel_text'] ) );
                    break;
                case 'feels-like':
                    /** Feels like icon. */
                    $output[] = $weather_datum[$p . '_feels_like_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-feels_like-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_feels_like_icon'],
                                'Feels like'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_feels_like_panel_text'] ) );
                    break;
                case 'pressure':
                    /** Pressure icon. */
                    $output[] = $weather_datum[$p . '_pressure_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-pressure-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_pressure_icon'],
                                'Pressure'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_pressure_panel_text'] ) );
                    break;
                case 'humidity':
                    /** Humidity icon. */
                    $output[] = $weather_datum[$p . '_humidity_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-humidity-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_humidity_icon'],
                                'Humidity'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_humidity_panel_text'] ) );
                    break;
                case 'cloudy':
                    /** Cloudy icon. */
                    $output[] = $weather_datum[$p . '_clouds_all_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-clouds_all-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_clouds_all_icon'],
                                'Cloudy'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_clouds_all_panel_text'] ) );
                    break;
                case 'wind-speed':
                    /** Wind speed icon. */
                    $output[] = $weather_datum[$p . '_wind_speed_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-wind_speed-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_wind_speed_icon'],
                                'Wind speed'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_wind_speed_panel_text'] ) );
                    break;
                case 'wind-direction':
                    /** Wind direction icon. */
                    $output[] = $weather_datum[$p . '_wind_deg_display_in_panel'] === 'icon' ?
                        sprintf(
                            '<div class="mdp-forcaster-elementor-wind_deg-icon">%s</div>',
                            $this->get_floating_icon(
                                $weather_datum[$p . '_wind_deg_icon'],
                                'Wind direction'
                            )
                        ) : sprintf( '<div class="mdp-floating-text">%s</div>', esc_html( $weather_datum[$p . '_wind_deg_panel_text'] ) );
                    break;
            }

        }

        return sprintf(
            '<div class="%s">%s</div>',
            esc_attr__( $class_box_icon ),
            implode( '', $output )
        );

    }

    /**
     * Rounds to an integer.
     *
     * @param $value - The number to be rounded. Required.
     *
     * @return float
     * @since 1.0.0
     * @access public
     *
     */
    private function get_rounding_numbers( $value ) {

        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** If we have value for the icon, prepare the markup for it. */
        if ( $settings['forcaster_round'] === 'yes' ) {
            $result = round( $value, 0 );
        }else{
            $result = $value;
        }

        return $result;

    }

    /**
     * Returns caption.
     *
     * @param $caption
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_hourly_caption( $caption ) {
        if ( !empty( $caption ) ) {
            return sprintf( '<span class="mdp-hourly-caption">%s</span>', $caption );
        }

        return '';

    }

    /**
     * Returns weather information.
     *
     * @param $repeater_item
     * @param $type - Element name.
     * @param $title - Text HTML attribute.
     * @param $rounding_numbers - The numerical value of the weather.
     * @param null $icon - Icon for graphic designation of weather data.
     *
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function get_whether_info( $repeater_item, $type, $title, $rounding_numbers, $icon = null ) {
        /** Get the current unit system. */
        $units = $this->weather_units();

        /** Contains text with units. */
        $units_text = '';

        /** We generate class for setting styles. */
        $class = 'mdp-hourly-' . str_replace( '_', '-', $type );

        $class_content_wrapper = sprintf(
            'mdp-forcaster-item-content mdp-forcaster-item-position-%s',
            esc_attr( $repeater_item['forcaster_hourly_' . $type . '_icon_position'] )
        );

        $class_text_box = sprintf(
            'mdp-hourly-text-box mdp-forcaster-item-position-%s',
            esc_attr( $repeater_item['forcaster_hourly_' . $type . '_caption_position'] )
        );

        /** Check the variable for void. */
        if ( isset( $units[$type] ) ) {
            $units_text = $units[$type];
        }

        $forcaster_caption = $repeater_item[ 'forcaster_hourly_' . $type . '_caption' ];

        $caption = $this->create_hourly_caption( $forcaster_caption );

        $numbers = sprintf( '<span>%s %s</span>', esc_html( $rounding_numbers ), esc_html__( $units_text, 'forcaster-elementor' ) );

        return sprintf(
            '<div class="mdp-forcaster-elementor-item %s" title="%s">
                        <span class="%s"> 
                            %s
                            <span class="%s">%s %s</span>
                        </span>
                    </div>',
                   esc_attr( $class ),
                   esc_attr( $title ),
                   esc_attr( $class_content_wrapper ),
                   $this->get_info_icon( $icon ),
                   esc_attr( $class_text_box ),
                   $caption,
                   $numbers
        );

    }

    /**
     * @param $settings
     * @param $type_data
     * @param $data
     *
     * @return string|void
     */
    public function get_location_info( $settings, $type_data, $data ){
        if( $settings["forcaster_weather_{$type_data}"] !== 'yes' ){ return; }

        $icon = $settings["forcaster_weather_{$type_data}_icon"];
        $data_icon = !empty( $icon ) ? $icon : '';

        // Added real name for countries
        if ( $type_data === 'country' ) {

            $countries = $this->get_country_options();
            $data = apply_filters( 'forecaster-country', $countries[ $data ] );

        }

        return sprintf(
            '<div class="mdp-hourly-%s">%s %s %s</div>',
            $type_data,
            $this->get_info_icon( $data_icon ),
            $settings["forcaster_weather_{$type_data}_caption"],
            esc_html( $data )
        );
    }

    /**
     * Lists hourly weather based on settings.
     *
     * @param $settings - Array with settings.
     * @param $data - An array containing weather data.
     * @param string $date_format - Specifies the time output format.
     * @param int $items - The number of lines to be display.
     *
     * @return string
     * @throws \Exception
     * @since 1.0.0
     * @access private
     */
    private function weather_clock( $settings, $data, $date_format = '%H : %M', $items = 5 ) {

        if ( ( !$data && !is_array( $data ) ) || $data['cod'] === '404' ) {
            return  esc_html__( 'Data not found.', 'forcaster-elementor' );
        }

        $time_format = $settings['hourly_time_format'] === 'wp-format' ?
            esc_attr( get_option( 'time_format' ) ) :
            esc_attr( $settings['hourly_time_format'] );

        if ( $data['cod'] === 401 ) {
            $is_editor = Plugin::instance()->editor->is_edit_mode();

            $invalid_key_message = esc_html( $data['message'] );
            return $is_editor ? $invalid_key_message : '';
        }

        /** Set default timezone. */
        date_default_timezone_set( $settings['timezone_country'] );

        /** Widget id. */
        $idSection = $this->get_id();

        /** Calculate the number of days displayed. */
        $count_day = 0;

        /** Depending on the type of display style, set the current date. */
        $now_date = date('Y-m-d' ).$this->weather_time();

        /** We generate a style class for the scroll container. */
        $class_scroll  = 'mdp-hourly-scroll';

        /** We generate a class of styles for the container depending on the type of weather. */
        $class_type    = 'mdp-hourly-type-info';

        /** We wrap the list in a scroll box. */

        $weather = sprintf(
            '<div class="mdp-forcaster-weather-box">%s %s %s</div>',
            $this->get_location_info( $settings, 'location', $data['city']['name'] ),
            $this->get_location_info( $settings, 'country', $data['city']['country'] ),
            $this->get_location_info( $settings, 'population', $data['city']['population'] )
        );

        $weather .= sprintf(
            '<div class="%s %s %s %s">',
            $class_scroll,
                   $class_type,
                   $class_scroll . '-' . $idSection,
                   $class_type . '-' . $idSection
        );

        /** We wrap the list in a container. */
        $weather .= sprintf(
            '<div class="mdp-hourly-container mdp-hourly-container-%s">',
                    esc_attr( $idSection )
        );

        if ( $settings['forcaster_floating_panel'] === 'yes' ) {
            $class_box_icon = sprintf( 'mdp-sticky-hourly mdp-sticky-hourly-%s', esc_attr( $idSection ) );
            $weather .= $this->floating_panel( $settings, $class_box_icon );
        }

        /** We display hourly weather in the cycle. */
        foreach( $data['list'] as $day => $value ) {

            /** Add a day to the current date during each cycle. */
            $date_day = new \DateTime( $now_date );
            $date_day->modify( '+'.$count_day.' day' );
            $date_day->format( 'U' );


            /** We set the condition depending on the selected type of layouts. */
            $condition = $day < $items;

            if( $condition ) {

                $dateTime = new \DateTime();

                /** Forms strings. */
                $weather .= sprintf(
                    '<div class="mdp-row-hourly mdp-row-hourly-%s">',
                    esc_attr( $idSection )
                );

                $weather_info = $settings['weather_hourly_data'];

                $show_icon = $settings['forcaster_floating_panel'] !== 'yes';

                foreach ( $weather_info as $weather_datum ) {
                    switch ( $weather_datum['forcaster_weather_hourly_info_type'] ) {
                        case 'date':
                            /** Determine whether to display the date. */
                            $weather .= sprintf(
                                '<div class="mdp-hourly-date">
                                            <span class="mdp-forcaster-item-content mdp-forcaster-item-position-%s">
                                                %s
                                                <span class="mdp-hourly-text-box mdp-forcaster-item-position-%s">
                                                    %s
                                                    <span class="mdp-forcaster-elementor-date-time-value">
                                                        <p>%s</p>
                                                        <p>%s</p>
                                                    </span>
                                                </span>
                                            </span>
                                        </div>',
                         esc_attr( $weather_datum['forcaster_hourly_date_icon_position'] ),
                                $show_icon ?
                                     $this->get_info_icon( $weather_datum['forcaster_hourly_date_icon'] ) :
                                    null,
                                esc_attr( $weather_datum['forcaster_hourly_date_caption_position'] ),
                                $this->create_hourly_caption( esc_html( $weather_datum['forcaster_hourly_date_caption'] ) ),
                                wp_date( 'F j', $value['dt'], $dateTime->getTimezone() ),
                                date( $time_format, $value['dt'] )
                            );
                            break;
                        case 'icon':
                            $style = $weather_datum['forcaster_hourly_visual_data_icon_style'];

                            $icon_url = UnityPlugin::get_url() . 'images/' . $style . $data['list'][$day]['weather'][0]['icon'] . '.png';
                            $weather .= sprintf(
                                '<div class="mdp-icon">
                                            <span class="mdp-forcaster-item-content mdp-forcaster-item-position-%s">
                                                %s
                                                <span class="mdp-hourly-text-box mdp-forcaster-item-position-%s">
                                                    %s
                                                    <img src="%s" class="mdp-%s-img-icon" alt="">
                                                </span>
                                            </span>
                                        </div>',
                         esc_attr( $weather_datum['forcaster_hourly_visual_data_icon_position'] ),
                                $show_icon ?
                                    $this->get_info_icon( $weather_datum['forcaster_hourly_visual_data_icon'] ) :
                                    null,
                                esc_attr( $weather_datum['forcaster_hourly_visual_data_caption_position'] ),
                                $this->create_hourly_caption( esc_html( $weather_datum['forcaster_hourly_visual_data_caption'] ) ),
                                $icon_url,
                                'hourly'
                            );
                            break;
                        case 'description':
                            $weather .= sprintf(
                                '<div class="mdp-hourly-description">
                                            <span class="mdp-forcaster-item-content mdp-forcaster-item-position-%s">
                                                %s
                                                <span class="mdp-hourly-text-box mdp-forcaster-item-position-%s">
                                                    %s
                                                    <span class="mdp-forcaster-elementor-description-text">%s</span>
                                                </span>
                                            </span>
                                        </div>',
                         esc_attr( $weather_datum['forcaster_hourly_description_icon_position'] ),
                                $show_icon ?
                                          $this->get_info_icon( $weather_datum['forcaster_hourly_description_icon'] ) :
                                          null,
                                esc_attr( $weather_datum['forcaster_hourly_description_caption_position'] ),
                                $this->create_hourly_caption( esc_html( $weather_datum['forcaster_hourly_description_caption'] ) ),
                                esc_html__( $value['weather'][0]['description'] )
                            );
                            break;
                        case 'temp':
                            $round_temp = $this->get_rounding_numbers( $value['main']['temp'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'temp',
                                'Temp',
                                $round_temp,
                                $show_icon ? $weather_datum['forcaster_hourly_temp_icon'] : null
                            );
                            break;
                        case 'feels-like':
                            $round_feels_like = $this->get_rounding_numbers( $value['main']['feels_like'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'feels_like',
                                'Feels like',
                                $round_feels_like,
                                $show_icon ? $weather_datum['forcaster_hourly_feels_like_icon'] : null
                            );
                            break;
                        case 'pressure':
                            $round_pressure = $this->get_rounding_numbers( $value['main']['pressure'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'pressure',
                                'Pressure',
                                $round_pressure,
                                $show_icon ? $weather_datum['forcaster_hourly_pressure_icon'] : null
                            );
                            break;
                        case 'humidity':
                            $round_humidity = $this->get_rounding_numbers( $value['main']['humidity'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'humidity',
                                'Humidity',
                                $round_humidity,
                                $show_icon ? $weather_datum['forcaster_hourly_humidity_icon'] : null
                            );
                            break;
                        case 'cloudy':
                            $round_clouds_all = $this->get_rounding_numbers( $value['clouds']['all'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'clouds_all',
                                'Cloudy',
                                $round_clouds_all,
                                $show_icon ? $weather_datum['forcaster_hourly_clouds_all_icon'] : null
                            );
                            break;
                        case 'wind-speed':
                            $round_wind_speed = $this->get_rounding_numbers( $value['wind']['speed'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'wind_speed',
                                'Wind speed',
                                $round_wind_speed,
                                $show_icon ? $weather_datum['forcaster_hourly_wind_speed_icon'] : null
                            );
                            break;
                        case 'wind-direction':
                            $round_wind_deg = $this->get_rounding_numbers( $value['wind']['deg'] );
                            $weather .= $this->get_whether_info(
                                $weather_datum,
                                'wind_deg',
                                'Wind direction',
                                $round_wind_deg,
                                $show_icon ? $weather_datum['forcaster_hourly_wind_deg_icon'] : null
                            );
                            break;
                    }

                }

                /** Close the line. */
                $weather .= '</div>';

                /** Increase the counter for the day. */
                $count_day++;

            }

        }

        /** Close the container. */
        $weather .= '</div>';

        /** Left button prefix. */
        $button_hourly_left  = 'mdp-button-left-hourly';

        /** Right button prefix. */
        $button_hourly_right = 'mdp-button-right-hourly';

        $button_hourly_hide = $settings['forcaster_hide_buttons'] === 'yes' ?
            'mdp-hourly-button-hide' : '';

        /** Html attribute. */
        $data_attribute      ='data-hourly' . esc_attr__( $idSection );

        $next_hourly_button = $settings['weather_hourly_next_btn_icon']['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $settings['weather_hourly_next_btn_icon']['value'] ) :
            '<i class="'.esc_attr( $settings['weather_hourly_next_btn_icon']['value'] ).'"></i>';

        $previous_hourly_button = $settings['weather_hourly_previous_btn_icon']['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $settings['weather_hourly_previous_btn_icon']['value'] ) :
            '<i class="'.esc_attr( $settings['weather_hourly_previous_btn_icon']['value'] ).'"></i>';

        $weather .= sprintf(
            '<button class="mdp-button-info %s %s %s" %s="-1">%s</button>',
           $button_hourly_hide,
                   $button_hourly_left,
                   $button_hourly_left . '-' . esc_attr( $idSection ),
                   $data_attribute,
                   $previous_hourly_button
        );
        $weather .= sprintf(
            '<button class="mdp-button-info %s %s %s" %s="1">%s</button>',
           $button_hourly_hide,
                   $button_hourly_right,
                   $button_hourly_right . '-' . esc_attr( $idSection ),
                   $data_attribute,
                   $next_hourly_button
        );


        /** Close the scroll box. */
        $weather .= '</div>';

        /** We remove the flow around the container. */
        $weather .= '<div style="clear: both;"></div>';


        $weather .= sprintf( "<script>
                jQuery( document ).ready( function( $ ) {
                    let distance = 135 * %s;
                    let box = $( '.mdp-hourly-container-%s' );
                    let stickyPanel = $( '.mdp-sticky-hourly-%s' );
                    let leftButton = $( '.mdp-button-left-hourly-%s' );
                    let rightButton = $( '.mdp-button-right-hourly-%s' );
                    
                    if ( stickyPanel.length === 0 ) {
                        leftButton.css( { 'left': 0 } );
                    }
                    
                    leftButton.on( 'click', function() {
                        box.stop().animate( {
                            scrollLeft: '+=' + ( distance * $( this ).data( 'hourly%s' ) )
                        } );
                    } );
                    rightButton.on( 'click', function() {
                        box.stop().animate( {
                            scrollLeft: '+=' + ( distance * $( this ).data( 'hourly%s' ) )
                        } );
                    } );
                } );
            </script>",
            esc_attr__( $settings['style_hourly_scrolling_items']['size'] ),
            esc_attr__( $idSection ),
            esc_attr__( $idSection ),
            esc_attr__( $idSection ),
            esc_attr__( $idSection ),
            esc_attr__( $idSection ),
            esc_attr__( $idSection )
        );

        return $weather;
    }

    /**
     * Displays the layouts in the given order.
     *
     * @param $settings - Array with settings.
     * @param $openweathermap
     * @throws \Exception
     */
    private function sorting_layout( $settings, $openweathermap ) {

        $quantity_items = $settings['style_hourly_quantity_items']['size'];

        /** The weather is displayed for every three hours. */
        echo $this->weather_clock(
            $settings,
            $openweathermap,
            '<p>%b %e</p><p>%H:%M</p>',
            $quantity_items
        );

    }


    /**
     * Determine site visitor IP
     *
     * @since 1.0.0
     * @access private
     **/
    private function getUserIP() {

        if ( isset( $_SERVER["HTTP_CF_CONNECTING_IP"] ) ) {
            $_SERVER['REMOTE_ADDR'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
            $_SERVER['HTTP_CLIENT_IP'] = $_SERVER["HTTP_CF_CONNECTING_IP"];
        }
        $client  = @$_SERVER['HTTP_CLIENT_IP'];
        $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
        $remote  = $_SERVER['REMOTE_ADDR'];

        if( filter_var( $client, FILTER_VALIDATE_IP ) ) {
            $ip = $client;
        }elseif( filter_var( $forward, FILTER_VALIDATE_IP ) ) {
            $ip = $forward;
        }else{
            $ip = $remote;
        }

        return $ip;
    }

    /**
     * Get auto location in object
     * @return mixed|object
     */
    private function get_auto_location() {

        $ip = $this->getUserIP();
        if( $ip === '::1'){
            $ip = @file_get_contents('https://api.ipify.org');
        }

        // Set default location
        $default_location = (object) array(
            'city' => 'Kyiv',
            'country_code' => 'UA',
        );

        if ( empty( $ip ) ) { return $default_location; }

        $get_location = wp_remote_get( "https://ipapi.co/{$ip}/json/" );
        if ( ! is_array( $get_location ) ) { return $default_location;}

        $get_location_body = wp_remote_retrieve_body( $get_location );
        return json_decode( $get_location_body );

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @throws \Exception
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {
        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        $is_auto_location = $settings['forcaster_auto_location'] === 'yes';

        if ( $is_auto_location ) {

            $auto_location = $this->get_auto_location();

        }

        /** Get Open Weather Key settings. */
        $OpenWeatherKey = Settings::get_instance()->options[ 'mdp_synopter_elementor_weather_settings' ];

        if ( ! empty( $OpenWeatherKey ) ) {

            /** We get weather data from an external source. Updated every 3 hours. */
            $openWeatherURL = sprintf(
         'https://api.openweathermap.org/data/2.5/forecast?q=%s,%s&units=%s&lang=%s&appid=%s',
        $is_auto_location ? urldecode( $auto_location->city ) : urldecode( $settings[ 'location_city' ] ),
                $is_auto_location ? urldecode( $auto_location->country_code ) : urldecode( $settings[ 'location_country' ] ),
                esc_attr( $settings[ 'location_units' ] ),
                esc_attr( $settings[ 'language_country' ] ),
                esc_attr( $OpenWeatherKey )
            );

            $json_openweathermap = wp_remote_get( $openWeatherURL );

            /** Decodes the JSON string. */
            $openweathermap = json_decode( $json_openweathermap['body'], true );

            /** Layout Order. */
            $this->sorting_layout( $settings, $openweathermap );

        }else{

            echo sprintf(
                '<p class="mpb-text-warning"><strong>%s</strong></p>',
                esc_html__( 'Warning: The Open Weather Key is not installed.', 'forcaster-elementor' )
            );

        }

    }

    /**
     * Filter for country name
     * @param $country
     *
     * @return string
     */
    public function filter_country( $country ) {

        if ( $country === 'Russian Federation' ) {

            return $country . esc_html__( ' (terrorist state)' );

        }

        return $country;

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/forcaster';

    }

}
